import { HttpClient } from '@angular/common/http';
import { Component, Inject, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog';
import { MatSnackBar } from '@angular/material/snack-bar';
import { ToastrService } from 'ngx-toastr';
import { DataService } from '../../../services/data.service';

@Component({
  selector: 'app-create-client',
  templateUrl: './create-client.component.html',
  styleUrls: ['./create-client.component.css']
})
export class CreateClientComponent implements OnInit {
  baseUrl: string = import.meta.env.NG_APP_BASE_API_URL;
  clientForm!: FormGroup;
  parentUnits: any[] = [];
  parentUnitsWithPaths: { id: string, name: string, path: string }[] = [];
  hardwareProfiles: any[] = [];
  ogLives: any[] = [];
  menus: any[] = [];
  templates: any[] = [];
  repositories: any[] = [];
  loading: boolean = false;
  displayedColumns: string[] = ['name', 'ip'];
  protected netifaceTypes = [
    { name: 'Eth0', value: 'eth0' },
    { name: 'Eth1', value: 'eth1' },
    { name: 'Eth2', value: 'eth2' }
  ];
  protected netDriverTypes = [
    { name: 'Generic', value: 'generic' }
  ];

  constructor(
    private fb: FormBuilder,
    private dialogRef: MatDialogRef<CreateClientComponent>,
    private http: HttpClient,
    private snackBar: MatSnackBar,
    private toastService: ToastrService,
    private dataService: DataService,
    @Inject(MAT_DIALOG_DATA) public data: any
  ) {}

  ngOnInit(): void {
    this.loadParentUnits();
    this.loadHardwareProfiles();
    this.loadOgLives();
    this.loadPxeTemplates();
    this.loadRepositories();
    this.loadMenus()
    this.initForm();
  }

  initForm(): void {
    this.clientForm = this.fb.group({
      organizationalUnit: [
        this.data.organizationalUnit ? this.data.organizationalUnit['@id'] : null,
        Validators.required
      ],
      name: ['', Validators.required],
      serialNumber: [''],
      netiface: null,
      netDriver: null,
      mac: ['', Validators.required],
      ip: ['', Validators.required],
      template: [null],
      hardwareProfile: [null],
      ogLive: [null],
      repository: [null],
      menu: [null]
    });
  }

  loadParentUnits(): void {
    this.loading = true;
    const url = `${this.baseUrl}/organizational-units?page=1&itemsPerPage=10000`;

    this.http.get<any>(url).subscribe(
      response => {
        this.parentUnits = response['hydra:member'];
        this.parentUnitsWithPaths = this.parentUnits.map(unit => ({
          id: unit['@id'],
          name: unit.name,
          path: this.dataService.getOrganizationalUnitPath(unit, this.parentUnits),
          repository: unit.networkSettings?.repository?.['@id'],
          hardwareProfile: unit.networkSettings?.hardwareProfile?.['@id'],
          ogLive: unit.networkSettings?.ogLive?.['@id'],
          menu: unit.networkSettings?.menu?.['@id']
        }));

        // 🚀 Ahora que los datos están listos, aplicamos la configuración inicial
        const initialUnitId = this.clientForm.get('organizationalUnit')?.value;
        if (initialUnitId) {
          this.setOrganizationalUnitDefaults(initialUnitId);
        }

        this.loading = false;
      },
      error => {
        console.error('Error fetching parent units:', error);
        this.loading = false;
      }
    );
  }

  getSelectedParentName(): string | undefined {
    const parentId = this.clientForm.get('organizationalUnit')?.value;
    return this.parentUnitsWithPaths.find(unit => unit.id === parentId)?.name;
  }

  loadHardwareProfiles(): void {
    this.dataService.getHardwareProfiles().subscribe(
      (data: any[]) => {
        this.hardwareProfiles = data;
        this.loading = false;
      },
      error => {
        console.error('Error fetching hardware profiles:', error);
        this.loading = false;
      }
    );
  }

  loadOgLives(): void {
    const url = `${this.baseUrl}/og-lives?page=1&itemsPerPage=30`;

    this.http.get<any>(url).subscribe(
      response => {
        this.ogLives = response['hydra:member'];
      },
      error => {
        console.error('Error fetching ogLives:', error);
      }
    );
  }

  loadPxeTemplates(): void {
    const url = `${this.baseUrl}/pxe-templates?page=1&itemsPerPage=10000`;

    this.http.get<any>(url).subscribe(
      response => {
        this.templates = response['hydra:member'];
      },
      error => {
        console.error('Error fetching PXE templates:', error);
      }
    );
  }

  loadMenus(): void {
    const url = `${this.baseUrl}/menus?page=1&itemsPerPage=10000`;

    this.http.get<any>(url).subscribe(
      response => {
        this.menus = response['hydra:member'];
      },
      error => {
        console.error('Error fetching menus:', error);
      }
    );
  }

  loadRepositories(): void {
    const url = `${this.baseUrl}/image-repositories?page=1&itemsPerPage=10000`;

    this.http.get<any>(url).subscribe(
      response => {
        this.repositories = response['hydra:member'];
      },
      error => {
        console.error('Error fetching ogLives:', error);
      }
    );
  }

  onParentChange(event: any): void {
    this.setOrganizationalUnitDefaults(event.value);
  }

  setOrganizationalUnitDefaults(unitId: string): void {
    const selectedUnit: any = this.parentUnitsWithPaths.find(unit => unit.id === unitId);
    if (selectedUnit) {
      this.clientForm.patchValue({
        repository: selectedUnit.repository || null,
        hardwareProfile: selectedUnit.hardwareProfile || null,
        ogLive: selectedUnit.ogLive || null,
        menu: selectedUnit.menu || null
      });
    }
  }

  onSubmit(): void {
    if (this.clientForm.valid) {
      const formData = this.clientForm.value;

      this.http.post(`${this.baseUrl}/clients`, formData).subscribe(
        (response) => {
          this.toastService.success('Cliente creado exitosamente', 'Éxito');
          this.dialogRef.close({
            client: response,
            organizationalUnit: formData.organizationalUnit,
          });
        },
        (error) => {
          this.toastService.error(error.error['hydra:description'], 'Error al crear el cliente');
        }
      );
    } else {
      this.toastService.error('Formulario inválido. Por favor, revise los campos obligatorios.', 'Error');
    }
  }

  onNoClick(): void {
    this.dialogRef.close();
  }
}
